from django.core.exceptions import ObjectDoesNotExist
from wagtail.wagtailcore.models import Site

from flags.models import Flag


def flag_enabled(request, key):
    site = Site.find_for_request(request)

    try:
        return site.flag_states.get(flag_id=key).enabled
    except ObjectDoesNotExist:
        pass

    try:
        return Flag.objects.get(key=key).enabled_by_default
    except ObjectDoesNotExist:
        return False


def flags_enabled(request, *keys):
    return all(flag_enabled(request, key) for key in keys)


def flag_disabled(request, key):
    return not flag_enabled(request, key)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0019_verbose_names_cleanup'),
    ]

    operations = [
        migrations.CreateModel(
            name='Flag',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('key', models.CharField(max_length=500)),
                ('enabled', models.BooleanField()),
                ('site', models.ForeignKey(to='wagtailcore.Site')),
            ],
        ),
        migrations.CreateModel(
            name='SiteSettings',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('default', models.BooleanField()),
                ('site', models.ForeignKey(to='wagtailcore.Site')),
            ],
        ),
    ]



from django import forms
from django.core import validators

from .models import Flag, FlagState


validate_flag = validators.RegexValidator(
    r'^[-a-zA-Z0-9_]+\Z',
    "Enter a valid flag consisting of letters, numbers, or underscores.",
    'invalid'
)


class SelectSiteForm(forms.Form):
    site_id = forms.IntegerField()


class FeatureFlagForm(forms.ModelForm):
    key = forms.CharField(validators=[validate_flag])

    class Meta:
        model = Flag
        fields = ('key', )


class FlagStateForm(forms.ModelForm):
    class Meta:
        model = FlagState
        fields = ('flag', 'enabled', 'site')
        widgets = {
            'flag': forms.widgets.HiddenInput(),
            'site': forms.widgets.HiddenInput(),
        }



from django.core.exceptions import ImproperlyConfigured
from django.http import Http404, HttpRequest, HttpResponse
from django.test import TestCase
from django.views.generic import View
from wagtail.wagtailcore.models import Page, Site
from wagtail.wagtailcore.views import serve as wagtail_serve

from flags.models import Flag
from flags.views import FlaggedViewMixin


class TestView(FlaggedViewMixin, View):
    def get(self, request, *args, **kwargs):
        return HttpResponse('ok')


class FlaggedViewMixinTestCase(TestCase):
    def setUp(self):
        self.flag_name = 'FLAGGED_VIEW_MIXIN_TEST_CASE'

    def request(self, path='/'):
        request = HttpRequest()

        request.method = 'GET'
        request.path = path
        request.META['SERVER_NAME'] = 'localhost'
        request.META['SERVER_PORT'] = 8000
        request.site = Site.objects.get(is_default_site=True)

        return request

    def test_no_flag_key_raises_improperly_configured(self):
        view = TestView.as_view()
        self.assertRaises(ImproperlyConfigured, view, self.request())

    def test_no_flag_acts_as_disabled(self):
        view = TestView.as_view(flag_name=self.flag_name)
        self.assertRaises(Http404, view, self.request())

    def test_flag_set_view_enabled(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)
        view = TestView.as_view(flag_name=self.flag_name)
        self.assertEqual(view(self.request()).status_code, 200)

    def test_flag_set_view_disabled(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=False)
        view = TestView.as_view(flag_name=self.flag_name)
        self.assertRaises(Http404, view, self.request())

    def test_fallback_view_function_disabled(self):
        def test_view_function(request, *args, **kwargs):
            return HttpResponse('fallback fn')

        view = TestView.as_view(
            flag_name=self.flag_name,
            fallback_view=test_view_function
        )

        response = view(self.request())
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'fallback fn')

    def test_fallback_view_function_enabled(self):
        def test_view_function(request, *args, **kwargs):
            return HttpResponse('fallback fn')

        Flag.objects.create(key=self.flag_name, enabled_by_default=True)
        view = TestView.as_view(
            flag_name=self.flag_name,
            fallback_view=test_view_function
        )

        response = view(self.request())
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'ok')

    def test_fallback_class_based_view(self):
        class OtherTestView(View):
            def get(self, request, *args, **kwargs):
                return HttpResponse('fallback cbv')

        view = TestView.as_view(
            flag_name=self.flag_name,
            fallback_view=OtherTestView.as_view()
        )

        response = view(self.request())
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'fallback cbv')

    def test_fallback_wagtail_serve(self):
        site = Site.objects.get(is_default_site=True)
        root = site.root_page
        page = Page(title='wagtail title', slug='title')
        root.add_child(instance=page)
        page.save()
        page.save_revision().publish()

        fail_through = lambda request: wagtail_serve(request, request.path)
        view = TestView.as_view(flag_name=self.flag_name,
                                fallback_view=fail_through)

        response = view(self.request(path='/title'))
        self.assertContains(response, '<title>wagtail title</title>')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def add_beta_flag(apps,schema_editor):
    Flag=apps.get_model('flags','flag')
    beta_notice = Flag(key='BETA_NOTICE', enabled_by_default=True)
    beta_notice.save()


class Migration(migrations.Migration):

    dependencies = [
        ('flags', '0005_flag_enabled_by_default'),
    ]

    operations = [
	migrations.RunPython(add_beta_flag)
    ]



try:
    from mock import Mock, patch
except ImportError:
    from mock import Mock, patch

from django.http import HttpRequest
from django.test import TestCase

from wagtail.wagtailcore.models import Site

from flags.models import Flag, FlagState
from flags.template_functions import flag_disabled, flag_enabled, flags_enabled


class FlagEnabledTestCase(TestCase):
    def setUp(self):
        self.flag_name = 'FLAG_ENABLED_TEST_CASE'
        self.site = Site.objects.get(is_default_site=True)
        self.request = HttpRequest()
        self.request.site = self.site

    def test_no_flag_should_be_disabled(self):
        self.assertFalse(flag_enabled(self.request, self.flag_name))

    def test_flag_default_false_disabled(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=False)
        self.assertFalse(flag_enabled(self.request, self.flag_name))

    def test_flag_default_true_enabled(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)
        self.assertTrue(flag_enabled(self.request, self.flag_name))

    def test_flag_false_disabled(self):
        flag = Flag.objects.create(key=self.flag_name)
        FlagState.objects.create(flag=flag, site=self.site, enabled=False)
        self.assertFalse(flag_enabled(self.request, self.flag_name))

    def test_flag_true_enabled(self):
        flag = Flag.objects.create(key=self.flag_name)
        FlagState.objects.create(flag=flag, site=self.site, enabled=True)
        self.assertTrue(flag_enabled(self.request, self.flag_name))

    def test_flag_for_other_site_disabled(self):
        flag = Flag.objects.create(key=self.flag_name)
        other_site = Site.objects.create(
            is_default_site=False,
            root_page_id=self.site.root_page_id
        )

        FlagState.objects.create(flag=flag, site=other_site, enabled=True)
        self.assertFalse(flag_enabled(self.request, self.flag_name))


class FlagDisabledTestCase(TestCase):
    def setUp(self):
        self.flag_name = 'FLAG_DISABLED_TEST_CASE'
        self.request = Mock()

    def test_flag_disabled_true(self):
        with patch(
            'flags.template_functions.flag_enabled',
            return_value=True
        ):
            self.assertFalse(flag_disabled(self.request, self.flag_name))

    def test_flag_disabled_false(self):
        with patch(
            'flags.template_functions.flag_enabled',
            return_value=False
        ):
            self.assertTrue(flag_disabled(self.request, self.flag_name))


class FlagsEnabledTestCase(TestCase):
    def setUp(self):
        self.flag_names = ('FOO', 'BAR', 'BAZ')
        self.request = Mock()

    def test_flags_enabled_all_true(self):
        with patch(
            'flags.template_functions.flag_enabled',
            side_effect=[True, True, True]
        ):
            self.assertTrue(flags_enabled(self.request, *self.flag_names))

    def test_flags_enabled_some_true(self):
        with patch(
            'flags.template_functions.flag_enabled',
            side_effect=[True, False, True]
        ):
            self.assertFalse(flags_enabled(self.request, *self.flag_names))



from django.conf.urls import url, include
from django.core.urlresolvers import resolve
from django.http import HttpResponse, Http404
from django.test import TestCase, RequestFactory, override_settings

from flags.urls import flagged_url
from flags.models import Flag


def view(request):
    return HttpResponse('view')


def fallback(request):
    return HttpResponse('fallback')


urlpatterns = [
    flagged_url('FLAGGED_URL', r'^url-true-no-fallback$', view,
                name='some-view', condition=True),
    flagged_url('FLAGGED_URL', r'^url-false-no-fallback$', view,
                name='some-view', condition=False),
    flagged_url('FLAGGED_URL', r'^url-true-fallback$', view,
                name='some-view', condition=True, fallback=fallback),
    flagged_url('FLAGGED_URL', r'^url-false-fallback$', view,
                name='some-view', condition=False, fallback=fallback),
]


@override_settings(ROOT_URLCONF=__name__)
class FlagCheckTestCase(TestCase):
    def setUp(self):
        self.flag_name = 'FLAGGED_URL'
        self.factory = RequestFactory()

    def test_flagged_url_true_no_fallback(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)

        request = self.factory.get('/url-true-no-fallback')
        resolved_view, args, kwargs = resolve('/url-true-no-fallback')
        response = resolved_view(request)

        self.assertContains(response, 'view')

    def test_flagged_url_true_no_fallback_false(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=False)

        request = self.factory.get('/url-true-no-fallback')
        resolved_view, args, kwargs = resolve('/url-true-no-fallback')

        with self.assertRaises(Http404):
            resolved_view(request)

    def test_flagged_url_false_no_fallback(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=False)

        request = self.factory.get('/url-false-no-fallback')
        resolved_view, args, kwargs = resolve('/url-false-no-fallback')
        response = resolved_view(request)

        self.assertContains(response, 'view')

    def test_flagged_url_false_no_fallback_true(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)

        request = self.factory.get('/url-false-no-fallback')
        resolved_view, args, kwargs = resolve('/url-false-no-fallback')

        with self.assertRaises(Http404):
            resolved_view(request)

    def test_flagged_url_true_fallback(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)

        request = self.factory.get('/url-true-fallback')
        resolved_view, args, kwargs = resolve('/url-true-fallback')
        response = resolved_view(request)

        self.assertContains(response, 'view')

    def test_flagged_url_true_fallback_false(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=False)

        request = self.factory.get('/url-true-fallback')
        resolved_view, args, kwargs = resolve('/url-true-fallback')
        response = resolved_view(request)

        self.assertContains(response, 'fallback')

    def test_flagged_url_false_fallback(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=False)

        request = self.factory.get('/url-false-fallback')
        resolved_view, args, kwargs = resolve('/url-false-fallback')
        response = resolved_view(request)

        self.assertContains(response, 'view')

    def test_flagged_url_false_fallback_false(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)

        request = self.factory.get('/url-false-fallback')
        resolved_view, args, kwargs = resolve('/url-false-fallback')
        response = resolved_view(request)

        self.assertContains(response, 'fallback')

    def test_flagged_url_include(self):
        with self.assertRaises(TypeError):
            flagged_url('MY_FLAG', r'^my_url/$', include([
                url(r'^my_included_url', view)
            ]))

    def test_flagged_url_not_callable(self):
        with self.assertRaises(TypeError):
            flagged_url('MY_FLAG', r'^my_url/$', 'string')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0019_verbose_names_cleanup'),
        ('flags', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='FlagState',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('enabled', models.BooleanField(default=False)),
            ],
        ),
        migrations.RemoveField(
            model_name='sitesettings',
            name='site',
        ),
        migrations.RemoveField(
            model_name='flag',
            name='enabled',
        ),
        migrations.RemoveField(
            model_name='flag',
            name='id',
        ),
        migrations.RemoveField(
            model_name='flag',
            name='site',
        ),
        migrations.AlterField(
            model_name='flag',
            name='key',
            field=models.CharField(max_length=255, serialize=False, primary_key=True),
        ),
        migrations.DeleteModel(
            name='SiteSettings',
        ),
        migrations.AddField(
            model_name='flagstate',
            name='flag',
            field=models.ForeignKey(to='flags.Flag'),
        ),
        migrations.AddField(
            model_name='flagstate',
            name='site',
            field=models.ForeignKey(to='wagtailcore.Site'),
        ),
    ]



from django.test import TestCase
from wagtail.wagtailcore.models import Site

from flags.models import Flag, FlagState
from flags.utils import init_missing_flag_states_for_site


class InitMissingFlagStatesTestCase(TestCase):
    def setUp(self):
        self.site = Site.objects.get(is_default_site=True)
        self.flag_names = ('FOO', 'BAR')
        for flag_name in self.flag_names:
            Flag.objects.create(key=flag_name)

    def get_test_flag_states(self):
        return FlagState.objects.filter(flag_id__in=self.flag_names)

    def test_none_existing(self):
        states = self.get_test_flag_states()
        self.assertFalse(states.exists())

        init_missing_flag_states_for_site(self.site)

        states = self.get_test_flag_states()
        self.assertEqual(states.count(), 2)
        self.assertFalse(FlagState.objects.get(flag_id='FOO').enabled)
        self.assertFalse(FlagState.objects.get(flag_id='BAR').enabled)

    def test_some_existing(self):
        FlagState.objects.create(flag_id='FOO', enabled=True, site=self.site)
        init_missing_flag_states_for_site(self.site)

        states = self.get_test_flag_states()
        self.assertEqual(states.count(), 2)
        self.assertTrue(FlagState.objects.get(flag_id='FOO').enabled)
        self.assertFalse(FlagState.objects.get(flag_id='BAR').enabled)



from flags.models import Flag, FlagState


def init_missing_flag_states_for_site(site):
    existing_flags = site.flag_states.values_list('flag', flat=True)
    missing_flags = Flag.objects.exclude(key__in=existing_flags)

    FlagState.objects.bulk_create([
        FlagState(
            site=site,
            flag=flag,
            enabled=flag.enabled_by_default
        ) for flag in missing_flags
    ])



from django.http import Http404
from django.utils.functional import wraps

from flags.template_functions import flag_enabled


def flag_check(flag_name, condition, fallback=None):
    """ Check that a given flag has the given condition (True/False).
    If the condition is not met, perform the fallback.
    """
    def decorator(func):
        def inner(request, *args, **kwargs):
            enabled = flag_enabled(request, flag_name)

            if (condition and enabled) or (not condition and not enabled):
                return func(request, *args, **kwargs)
            elif fallback is not None:
                return fallback(request, *args, **kwargs)
            else:
                raise Http404

        return wraps(func)(inner)

    return decorator


def flag_required(flag_name, fallback_view=None, pass_if_set=True):
    return flag_check(flag_name, pass_if_set, fallback=fallback_view)



from django.db import models
from wagtail.wagtailcore.models import Site


class Flag(models.Model):
    key = models.CharField(max_length=255, primary_key=True)
    enabled_by_default = models.BooleanField(default=False)

    def __str__(self):
        return self.key


class FlagState(models.Model):
    flag = models.ForeignKey(Flag, related_name='states')
    site = models.ForeignKey(Site, related_name='flag_states')
    enabled = models.BooleanField(default=False)

    class Meta:
        unique_together = ('flag', 'site')



try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock

from django.http import Http404, HttpRequest, HttpResponse
from django.test import TestCase

from flags.decorators import flag_check
from flags.models import Flag


class FlagCheckTestCase(TestCase):
    def setUp(self):
        self.flag_name = 'FLAG_REQUIRED_TEST_CASE'

        self.request = HttpRequest()
        self.request.META['SERVER_NAME'] = 'localhost'
        self.request.META['SERVER_PORT'] = 8000

        self.view = Mock(__name__='view')

    def test_decorated_no_flag_exists(self):
        decorated = flag_check(self.flag_name, True)(self.view)
        self.assertRaises(Http404, decorated, self.request)
        self.assertEqual(self.view.call_count, 0)

    def test_decorated_flag_disabled(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=False)
        decorated = flag_check(self.flag_name, True)(self.view)
        self.assertRaises(Http404, decorated, self.request)
        self.assertEqual(self.view.call_count, 0)

    def test_decorated_flag_enabled(self):
        def view(request):
            return HttpResponse('ok')

        Flag.objects.create(key=self.flag_name, enabled_by_default=True)
        decorated = flag_check(self.flag_name, True)(view)
        response = decorated(self.request)
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'ok')

    def test_fallback_view(self):
        def fallback(request):
            return HttpResponse('fallback')

        decorator = flag_check(self.flag_name, True, fallback=fallback)
        decorated = decorator(self.view)
        response = decorated(self.request)
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'fallback')

    def test_pass_if_not_set_no_flag_exists(self):
        def view(request):
            return HttpResponse('ok')

        decorated = flag_check(self.flag_name, False)(view)
        response = decorated(self.request)
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'ok')

    def test_pass_if_not_set_disabled(self):
        def view(request):
            return HttpResponse('ok')

        Flag.objects.create(key=self.flag_name, enabled_by_default=False)
        decorated = flag_check(self.flag_name, False)(view)
        response = decorated(self.request)
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'ok')

    def test_pass_if_not_set_enabled(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)
        decorated = flag_check(self.flag_name, False)(self.view)
        self.assertRaises(Http404, decorated, self.request)
        self.assertEqual(self.view.call_count, 0)

    def test_pass_if_not_set_fallback_view(self):
        Flag.objects.create(key=self.flag_name, enabled_by_default=True)

        def fallback(request):
            return HttpResponse('fallback')

        decorator = flag_check(
            self.flag_name,
            False,
            fallback=fallback,
        )

        decorated = decorator(self.view)
        response = decorated(self.request)
        if isinstance(response.content, str):
            content = response.content
        else:
            content = bytes.decode(response.content)
        self.assertEqual(content, 'fallback')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def fix_beta_notice(apps, schema_editor):
    Flag = apps.get_model('flags','flag')

class Migration(migrations.Migration):

    dependencies = [
        ('flags', '0004_remove_flag_hidden'),
    ]

    operations = [
        migrations.AddField(
            model_name='flag',
            name='enabled_by_default',
            field=models.BooleanField(default=False),
        ),

    ]



from django.conf.urls import include, url
from django.core.urlresolvers import reverse
from wagtail.wagtailadmin.menu import MenuItem
from wagtail.wagtailcore import hooks

from flags import views


@hooks.register('register_settings_menu_item')
def register_flags_menu():
    return MenuItem('Flags', reverse('flagadmin:select_site'),
                    classnames='icon icon-tag', order=10000)


@hooks.register('register_admin_urls')
def register_flag_admin_urls():
    return [
        url(r'^flags/',
            include([
                url(r'^$', views.select_site, name='select_site'),
                url(r'^(\d+)/$', views.index, name='list'),
                url(r'^(\d+)/save/$', views.save, name='save'),
                url(r'^(\w+)/delete/$', views.delete, name='delete'),
                url(r'^create/$', views.create, name='create'),
            ], namespace='flagadmin'))
    ]



from django.db import IntegrityError
from django.test import TestCase
from wagtail.wagtailcore.models import Site

from flags.models import Flag, FlagState


class FlagTestCase(TestCase):
    def test_default_disabled(self):
        flag = Flag.objects.create(key='FOO')
        self.assertFalse(flag.enabled_by_default)


class FlagStateTestCase(TestCase):
    def setUp(self):
        default_site = Site.objects.get(is_default_site=True)
        self.site1 = Site.objects.create(
            hostname='host1',
            root_page_id=default_site.root_page_id
        )
        self.site2 = Site.objects.create(
            hostname='host2',
            root_page_id=default_site.root_page_id
        )

    def test_multiple_sites(self):
        flag = Flag.objects.create(key='FOO')
        FlagState.objects.create(flag=flag, site=self.site1)

        try:
            FlagState.objects.create(flag=flag, site=self.site2)
        except IntegrityError:
            self.fail('different sites can have different flag states')

    def test_one_flag_state_per_site(self):
        flag = Flag.objects.create(key='FOO')
        FlagState.objects.create(flag=flag, site=self.site1)
        with self.assertRaises(IntegrityError):
            FlagState.objects.create(flag=flag, site=self.site1)



from __future__ import absolute_import, unicode_literals

import os

SECRET_KEY = 'not needed'

DATABASES = {
    'default': {
        'ENGINE': os.environ.get(
            'DATABASE_ENGINE',
            'django.db.backends.sqlite3'
        ),
        'NAME': os.environ.get('DATABASE_NAME', 'flags'),
        'USER': os.environ.get('DATABASE_USER', None),
        'PASSWORD': os.environ.get('DATABASE_PASS', None),
        'HOST': os.environ.get('DATABASE_HOST', None),

        'TEST': {
            'NAME': os.environ.get('DATABASE_NAME', None),
        },
    },
}

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'wagtail.wagtailcore',
    'flags',
)

TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'APP_DIRS': True,
}]



from django.conf.urls import url

from flags.decorators import flag_check


def flagged_url(flag_name, regex, view, kwargs=None, name=None,
                condition=True, fallback=None):
    """ Make a URL depend on the state of a feature flag """

    if callable(view):
        flagged_view = flag_check(flag_name,
                                  condition,
                                  fallback=fallback)(view)
        return url(regex, flagged_view, kwargs=kwargs, name=name)

    elif isinstance(view, (list, tuple)):
        # XXX: For right now, we don't support include()
        raise TypeError('Flagged include() is not supported')

    else:
        raise TypeError('view must be a callable')



from setuptools import find_packages, setup

install_requires = [
    'Django>=1.8,<1.11',
    'wagtail>=1.6,<1.9',
]


testing_extras = [
    'mock>=2.0.0',
    'coverage>=3.7.0',
    'flake8>=2.2.0',
]


setup(
    name='wagtail-flags',
    url='https://github.com/cfpb/wagtail-flags',
    author='CFPB',
    author_email='tech@cfpb.gov',
    license='CC0',
    version='1.0.1',
    include_package_data=True,
    packages=find_packages(),
    install_requires=install_requires,
    extras_require={
        'testing': testing_extras,
    },
    classifiers=[
        'License :: Public Domain',
        'License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication'
    ]
)



from django import template

from flags.template_functions import (
    flag_enabled as base_flag_enabled,
    flags_enabled as base_flags_enabled,
    flag_disabled as base_flag_disabled
)


register = template.Library()


# @register.simple_tag(takes_context=True)
@register.assignment_tag(takes_context=True)
def flag_enabled(context, key):
    request = context['request']
    return base_flag_enabled(request, key)


# @register.simple_tag(takes_context=True)
@register.assignment_tag(takes_context=True)
def flags_enabled(context, *keys):
    request = context['request']
    return all(base_flags_enabled(request, key) for key in keys)


# @register.simple_tag(takes_context=True)
@register.assignment_tag(takes_context=True)
def flag_disabled(context, key):
    request = context['request']
    return base_flag_disabled(request, key)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flags', '0006_auto_20151217_2003'),
    ]

    operations = [
        migrations.AlterField(
            model_name='flagstate',
            name='flag',
            field=models.ForeignKey(related_name='states', to='flags.Flag'),
        ),
        migrations.AlterField(
            model_name='flagstate',
            name='site',
            field=models.ForeignKey(related_name='flag_states', to='wagtailcore.Site'),
        ),
        migrations.AlterUniqueTogether(
            name='flagstate',
            unique_together=set([('flag', 'site')]),
        ),
    ]



from django.http import HttpRequest
from django.template import Context, Template
from django.test import TestCase

from wagtail.wagtailcore.models import Site

from flags.models import Flag


class FlagsTemplateTagsTestCase(TestCase):

    def setUp(self):
        self.flag_names = ('ONE', 'TWO', 'THREE')
        self.site = Site.objects.get(is_default_site=True)
        self.request = HttpRequest()
        self.request.site = self.site

    def render_template(self, string, context=None):
        context = context or {'request': self.request}
        context = Context(context)
        return Template(string).render(context)

    def test_flag_enabled_disabled(self):
        rendered = self.render_template(
            '{% load feature_flags %}'
            '{% flag_enabled "TEMPLATETAG_TEST"  as test_flag %}'
            '{% if test_flag %}'
            'flag enabled'
            '{% else %}'
            'flag disabled'
            '{% endif %}'
        )
        self.assertEqual(rendered, 'flag disabled')

    def test_flag_enabled_enabled(self):
        Flag.objects.create(key='TEMPLATETAG_TEST', enabled_by_default=True)
        rendered = self.render_template(
            '{% load feature_flags %}'
            '{% flag_enabled "TEMPLATETAG_TEST"  as test_flag %}'
            '{% if test_flag %}'
            'flag enabled'
            '{% else %}'
            'flag disabled'
            '{% endif %}'
        )
        self.assertEqual(rendered, 'flag enabled')

    def test_flag_disabled_disabled(self):
        # Disabled can also mean non-existent
        rendered = self.render_template(
            '{% load feature_flags %}'
            '{% flag_disabled "TEMPLATETAG_TEST"  as test_flag %}'
            '{% if test_flag %}'
            'flag disabled'
            '{% else %}'
            'flag enabled'
            '{% endif %}'
        )
        self.assertEqual(rendered, 'flag disabled')

    def test_flag_disabled_enabled(self):
        Flag.objects.create(key='TEMPLATETAG_TEST', enabled_by_default=True)
        rendered = self.render_template(
            '{% load feature_flags %}'
            '{% flag_disabled "TEMPLATETAG_TEST"  as test_flag %}'
            '{% if test_flag %}'
            'flag disabled'
            '{% else %}'
            'flag enabled'
            '{% endif %}'
        )
        self.assertEqual(rendered, 'flag enabled')

    def test_flags_enabled_all_disabled(self):
        rendered = self.render_template(
            '{% load feature_flags %}'
            '{% flags_enabled "ONE" "TWO" "THREE"  as test_flag %}'
            '{% if test_flag %}'
            'all flags enabled'
            '{% else %}'
            'flags disabled'
            '{% endif %}'
        )
        self.assertEqual(rendered, 'flags disabled')

    def test_flags_enabled_some_enabled(self):
        Flag.objects.create(key='ONE', enabled_by_default=True)
        Flag.objects.create(key='TWO', enabled_by_default=True)
        rendered = self.render_template(
            '{% load feature_flags %}'
            '{% flags_enabled "ONE" "TWO" "THREE"  as test_flag %}'
            '{% if test_flag %}'
            'all flags enabled'
            '{% else %}'
            'flags disabled'
            '{% endif %}'
        )
        self.assertEqual(rendered, 'flags disabled')

    def test_flags_enabled_all_enabled(self):
        Flag.objects.create(key='ONE', enabled_by_default=True)
        Flag.objects.create(key='TWO', enabled_by_default=True)
        Flag.objects.create(key='THREE', enabled_by_default=True)
        rendered = self.render_template(
            '{% load feature_flags %}'
            '{% flags_enabled "ONE" "TWO" "THREE"  as test_flag %}'
            '{% if test_flag %}'
            'all flags enabled'
            '{% else %}'
            'flags disabled'
            '{% endif %}'
        )
        self.assertEqual(rendered, 'all flags enabled')



from django.core.exceptions import ImproperlyConfigured
from django.forms import modelformset_factory
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect, render
from django.views.generic import TemplateView

from wagtail.wagtailcore.models import Site

from .decorators import flag_required
from .forms import FeatureFlagForm, FlagStateForm, SelectSiteForm
from .models import Flag, FlagState
from .utils import init_missing_flag_states_for_site


def select_site(request):
    if request.method == 'POST':
        form = SelectSiteForm(request.POST)

        if form.is_valid():
            site_id = form.cleaned_data['site_id']
            return redirect('flagadmin:list', site_id)
    else:
        default_site = Site.objects.all().filter(is_default_site=True).get()
        return redirect('flagadmin:list', (default_site.id),)


def create(request):
    if request.method == 'POST':
        form = FeatureFlagForm(request.POST)
        if form.is_valid():
            flag = Flag(key=form.cleaned_data['key'])
            flag.save()
            return redirect('flagadmin:select_site')
    else:
        form = FeatureFlagForm()

    context = dict(form=form)
    return render(request, 'flagadmin/flags/create.html', context)


def index(request, site_id):
    sites = Site.objects.all()
    selected_site = Site.objects.get(pk=site_id)

    init_missing_flag_states_for_site(selected_site)
    FlagStateFormSet = modelformset_factory(
        FlagState,
        form=FlagStateForm,
        extra=0
    )

    flagstate_forms = FlagStateFormSet(
        queryset=selected_site.flag_states.all()
    )

    context = {
        'selected_site': selected_site,
        'sites': sites,
        'flagforms': flagstate_forms,
    }

    return render(request, 'flagadmin/index.html', context)


def save(request, site_id):
    if request.method == 'POST':
        selected_site = Site.objects.get(pk=site_id)
        FlagStateFormSet = modelformset_factory(
            FlagState,
            form=FlagStateForm,
            extra=0
        )

        formset = FlagStateFormSet(request.POST)
        if formset.is_valid():
            formset.save()

        # this doesn't address what happens if the formset is invalid
        # index+save should probably be refactored as a single CBV
        return redirect('flagadmin:list', (selected_site.id),)


def delete(request, flag_id):
    flag = get_object_or_404(Flag, pk=flag_id)

    if request.method == 'POST':
        flag.delete()
        return redirect('flagadmin:select_site')

    context = dict(flag_id=flag.key)
    return render(request, 'flagadmin/flags/delete.html', context)


class FlaggedViewMixin(object):
    flag_name = None
    fallback_view = None
    pass_if_set = True

    def dispatch(self, request, *args, **kwargs):
        if self.flag_name is None:
            raise ImproperlyConfigured(
                "FlaggedViewMixin requires a 'flag_name' argument."
            )

        super_dispatch = super(FlaggedViewMixin, self).dispatch

        decorator = flag_required(
            self.flag_name,
            fallback_view=self.fallback_view,
            pass_if_set=self.pass_if_set
        )

        return decorator(super_dispatch)(request, *args, **kwargs)


class FlaggedTemplateView(FlaggedViewMixin, TemplateView):
    pass
